//The same way LSTMClassifier related to MLPClassifier, LSTMRegressor and MLPRegressor are similar. A recurrent feedback network is maping input time series (instead of input points) to a target point. Let's do an example

//1. execute the codeblock below
(
~xypoint = Buffer.alloc(s,2);
~pointOut = Buffer.alloc(s,10);
~entry = FluidDataSet(s);
~gestureXY = Buffer(s);
~count = 0;
~series = FluidDataSeries(s);
~points = FluidDataSet(s);
~lstmr = FluidLSTMRegressor(s,[15, 5],50);
~makingEntries = true;

SynthDef(\writer, {var trig = Impulse.kr(50); FluidDataSetWr.kr(~entry, buf: ~xypoint, idNumber: PulseCount.kr(trig),trig:trig)}).send;

w = Window("LSTMRegressor", Rect(top: Window.screenBounds.height-400, width: 800,height: 400)).front;

~ms = MultiSliderView(w,Rect(top: 0,width: 400, height: 300)).size_(10).elasticMode_(1).isFilled_(1).action_{
	arg ms;
	if (~makingEntries, {~pointOut.setn(0,ms.value)});
}.value_(0.5.dup(10));

Button(w, Rect(10, 310, 100, 20)).states_([[\makingEntry],[\querying]]).action_{|x|~makingEntries = x.value == 0};

Button(w, Rect(10, 340, 100, 20)).states_([["press to fit"],["stop fitting"]]).action_{
	|x|switch (x.value,
		0, {~keeponfitting.stop},
		1, {~keeponfitting = Routine{var cond = Condition.new();
			~lstmr.fit(~series,~points,{|x|{~error.string_(" %".format(x))}.defer; cond.unhang;});
			cond.hang;
}.loop.play;})};

~error = TextView(w,Rect(130,340,100,20)).string_(\unfitted).editable_(false);

Button(w, Rect(250, 340, 100, 20)).states_([["press to reseed"]]).action_{~lstmr.clear; ~error.string_(\unfitted)};

StaticText(w, Rect(10,370,200,20)).string_("network structure:");

TextField(w,Rect(130,370,100,20)).string_("[15,5]").action_{|x|~lstmr.hiddenLayers = x.value.asArray; ~error.string_(\unfitted)};

FluidPlotter(w, Rect(left: 400, width: 400, height: 400), standalone: false, mouseMoveAction: {|...in|
	if (in[5].notNil, {
		//clear the ds
		~entry.clear;
		in[0].dict = Dictionary.newFrom(["cols",2,"data",Dictionary.newFrom([])]);
		//write the value to buf
		~xypoint.setn(0,in[[1,2]]);
		//start the writer function that has a constant frequency
		~timedWriter = Synth(\writer);
	}, {
		if (in[4].notNil, {
			//stop the writer function
			~timedWriter.free;
			//transfer to buf
			~entry.toBuffer(~gestureXY,action: {
				if (~makingEntries, {
					//make an entry in the FluidDataSeries with a given identifier
					~series.addSeries(~count.asSymbol, ~gestureXY);
					//and make an entry in the FluidDataSet with the same identifier
					~points.addPoint(~count.asSymbol, ~pointOut);
				},{
					~lstmr.predictSeries(~gestureXY, ~pointOut, {
						~pointOut.getn(0,10,{
							arg prediction;
							defer{
								~ms.value_(prediction);
							}
						});
					});
				});

			});
		}, {
			~xypoint.setn(0,in[[1,2]]);
			in[0].addPoint_(~count.asSymbol,in[1],in[2]);
			~count = ~count + 1;
		});
	});
})
)

// 2. do this sequence 3 times:
// 2.1 - make a shape on the multislider on the left
// 2.2 - draw a few instances of a gesture in the FluidPlotter you would like to associate with this data

// 3. change to query mode

// 4. fit the data until you have an error you can trust

// 5. draw a gesture in the FluidPlotter: an interpolated set of values appear in the multislider

// 6. like MLPRegressor, you can play with the network's architecture. Retraining will be needed